#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 02/05/2001
#################################################################################
#
# Mail - Script part of Webchat designed to compose, retreive, view and manage mail
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Date;						# Date processing
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table creation


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash
my %mail;							# Mail hash

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashMail($Database, \%mail);	# Retreive mail hash

	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		# Display title
		print "$mail{'TagTitle'}\n";

		# Decisions
		if (defined($param{$mail{'TxtMailDelete'}}))	# Remove mail
		{ MailPurge($Database, $cgi, \%param, \%mail); }
		elsif (defined($param{'SEND'}))			# Sender
		{ MailSender($Database, $cgi, \%param, \%mail); }
		elsif (defined($param{'MID'}))			# Viewer
		{ MailViewer($Database, $cgi, \%param, \%mail); }
		elsif (defined($param{'LIST'}))			# Mail listing
		{ MailList($Database, $cgi, \%param, \%mail); }
		else						# Composer
		{ MailComposer($Database, $cgi, \%param, \%mail); }

	}

	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# MailComposer
#
# Allows for the composition of mail to be sent out to the designated person.
# Since the list of puppets can get long, this will requires a textfield.

sub MailComposer						# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $Database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $body;						# Body of message

	my $puppets;						# Display of drop down
	my @puppets;						# List of puppets


	#####################
	# Program area

	# Retreive puppet list
	Webchat::Database::GetListPuppetYours($Database, \@puppets, $param->{'USER'});

	# Determine values
	# Reply
	if (defined($param->{'REPLY'}))
	{
		# Retreive information
		($_,
		 $from, 
		 $rcpt, 
		 $subj, 
		 $time, 
		 $body) = Webchat::Database::DataGetPuppeteerMail($Database, $param->{'REPLY'});

		# Formatted time
		$time = Webchat::Date::GetDate($time);
		
		# Inverse values
		$_ = $rcpt;
		$rcpt = $from;
		$from = $_;

		# Message message row
		$body =~ s/^(.)/> $1/gm;
		$body =~ s/^(.)/$rcpt Wrote on $time:\n$1/s;
	}

	# Normal message
	else
	{
		# Determine values
		$from = (defined($param->{'MFROM'})) ? $param->{'MFROM'} : $puppets[0];
		$rcpt = (defined($param->{'MRCPT'})) ? $mail->{'TxtMailInvalid'} : '';
		$subj = (defined($param->{'MSUBJ'})) ? $param->{'MSUBJ'} : '';
		$body = (defined($param->{'MBODY'})) ? $param->{'MBODY'} : '';
	}


	# Determine puppets
	$puppets = $cgi->popup_menu('MFROM', \@puppets, $from);


	# Display title
	print "$mail->{'TagTitleComposer'}\n";

	# Display area
	# Start of form, table
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);

	# Header
	Webchat::Table::MakeSingle($cgi, $mail->{'TagMailHeader'});
	Webchat::Table::MakeValid($cgi, $mail->{'TagMailFrom'}, $puppets);
	Webchat::Table::MakeValid($cgi, $mail->{'TagMailTo'}, $cgi->textfield('MRCPT',$rcpt,25,30));
	Webchat::Table::MakeValid($cgi, $mail->{'TagMailSubject'}, $cgi->textfield('MSUBJ', $subj,25,250));

	# Body
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeSingle($cgi, $mail->{'TagMailMessage'});
	Webchat::Table::MakeSingle($cgi, "<TEXTAREA NAME=\"MBODY\", ROWS=\"10\", COLS=\"60\", WRAP=\"SOFT\">$body</TEXTAREA>");


	# Submit and clear
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeValid($cgi, $cgi->submit($mail->{'TxtMailSend'}), $cgi->reset($mail->{'TxtMailReset'}));

	Webchat::Table::MakeBottom($cgi);
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'SEND', 'True');
	print $cgi->end_form(), "\n";
}


#####################
# MailList
#
# Shows a listing of all currently received emails, giving an option to delete, view, reply 
# to them in some matter or the other.

sub MailList							# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $Database = shift;					# Database Handle
	my $Statement;						# Database Statement

	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $id;							# Identification number;
	my $from;						# Origin of message
	my $subj;						# Subject of message
	my $time;						# Time of posting

	my $color;						# Selected color
	my $count    = 0;					# Arbitrary counter
	my $url      = $cgi->url();				# Current location
	my $inline;						# Inline parameter list


	#####################
	# Program area

	# Data gathering
	# Prepare and execute
	$Statement = $Database->prepare("SELECT MailIDNumber,
		 MailFrom,
		 MailSubject,
		 MailTimestamp
		FROM Mail
		WHERE PuppeteerLogin=?
		ORDER BY MailTimestamp");
	$Statement->execute($param->{'USER'});


	# Initial display
	print "$mail->{TagTitleListing}\n<BR>\n";
	print "<CENTER><TABLE WIDTH=\"500\">\n",
	      "\t<TR BGCOLOR=\"$mail->{SetOptColor}\">\n",
	      "\t\t<TD>$mail->{TagListFrom}</TD>\n",
	      "\t\t<TD>$mail->{TagListSubject}</TD>\n",
	      "\t\t<TD>$mail->{TagListDate}</TD>\n",
	      "\t</TR>\n";

	# Retreive and display
	while (($id, $from, $subj, $time) = $Statement->fetchrow())
	{
		# Increment
		$count++;

		# Determine values
		# Generate inline
		$inline = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'MID', $id);

		# Determine colour
		$color = (($count % 2) != 0) ? $mail->{'SetSecColor'} : $mail->{'SetPriColor'};

		# Information display
		# Data display
		print "\t<TR BGCOLOR=\"$color\">\n",
		      "\t\t<TD><B><A HREF=\"$url$inline\">$from</A></B></TD>\n",
		      "\t\t<TD>$subj</TD>\n",
		      "\t\t<TD>", Webchat::Date::GetDate($time), "</TD>\n",
		      "\t</TR>\n";
	}


	# Determine values
	$inline = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});

	# Summary display
	print "\t<TR BGCOLOR=\"$mail->{SetOptColor}\">\n",
	      "\t\t<TD>$mail->{'TagListTotal'} $count</TD>\n",
	      "\t\t<TD COLSPAN=\"2\" ALIGN=\"RIGHT\"><A HREF=\"$url$inline\">$mail->{TagLinkCompose}</A></TD>\n",
	      "\t</TR>\n";


	# End of table
	print "</TABLE><CENTER>\n<BR>\n";

	# End execution
	$Statement->finish();
}

#####################
# Mail Purge
#
# Allows for the purging of vile messages from the list of known messages.   Simple and easy
# to use.

sub MailPurge							# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $Database = shift;					# Database Handle
	my $Statement;						# Database Statement

	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $id;							# Identification number;


	#####################
	# Program area

	# Retreive id
	$id = $param->{'OID'};

	# Parameter handling
	delete $param->{'OID'};
	delete $param->{$mail->{'TagMailSent'}};	

	
	# Remove entry
	$Database->do("DELETE FROM Mail 
		WHERE MailIDNumber=?
		  AND PuppeteerLogin=?", {},

	 $id,
	 $param->{'USER'});

	
	# Call listing
	MailList($Database, $cgi, $param, $mail);
}



#####################
# MailSender
#
# Allows for composed message to be sent out to the designated person.

sub MailSender							# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $Database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $body;						# Body of message

	my $id;							# ID Number of message
	my $notice;						# Notice message

	my $email;						# Email address
	my $name;						# Name of target
	my $target;						# Display of drop down

	my %system;						# Systems hash


	#####################
	# Program area

	# Data retreival
	# Retreive parts
	$from = $param->{'MFROM'};	
	$rcpt = $param->{'MRCPT'};
	$subj = $param->{'MSUBJ'};
	$body = $param->{'MBODY'};

	# Determine time
	$time = time;

	# Determine puppeteer
	($target) = Webchat::Database::DataGetPuppeteerLogin($Database, $param->{'MRCPT'});


	# Determine action
	# Processing
	if (defined($target))
	{
		# Retreive full name
		($name)  = Webchat::Database::DataGetPuppeteerName($Database, $target);
		($email) = Webchat::Database::DataGetPuppeteerEmail($Database, $target);

		# Insert into database
		$Database->do("INSERT INTO Mail
			(MailIDNumber,
			 PuppeteerLogin,
			 MailFrom,
			 MailRcpt,
			 MailSubject,
			 MailTimestamp,
			 MailContents)
			VALUES(NULL,?,?,?,?,?,?)", {},

		 $target,
		 $from,
		 $rcpt,
		 $subj,
		 $time,
		 $body);


		# Final preperations
		# Retreive values
		($id)     = Webchat::Database::DataGetPuppeteerMailID($Database, $target, $time);
		($notice) = Webchat::Database::DocumentGetMailNotice($Database);
				
		# Build link
		my $url    = $cgi->url();
		my $inline = Webchat::Param::EmbedInline($Database, $cgi, 'MID', $id, 'MRCPT', $rcpt, 'MFROM', $from, 'MSUBJ', $subj);


		# Compose and send email
		# Link to hash
		Webchat::Database::GetHashSystem($Database, \%system);

		# Message information
		if (defined($notice))
		{
			# Go though all keywords and replace with real value
			$notice =~ s/AUTHSTRING/$url$inline/g;
			$notice =~ s/FULLNAME/$name/g;
			$notice =~ s/ORIGIN/$from/g;
			$notice =~ s/RETURNEMAIL/$system{'SetInfoContactAddress'}/g;
			$notice =~ s/SERVERNAME/$system{'SetInfoServer'}/g;
			$notice =~ s/SUBJECT/$subj/g;


			# Define and open sendmail
			open(SENDMAIL, "|$system{'LocSendmail'} -oi -t")
				or die("Cannot fork sendmail: $!");

			# Send out invitation
			print SENDMAIL "From: $system{'SetInfoContactName'} <$system{'SetInfoContactAddress'}>\n";
			print SENDMAIL "To: $name <$email>\n";
			print SENDMAIL "Subject: You have mail from: $from\n\n";
			print SENDMAIL "$notice\n";
			close(SENDMAIL);
		}

		# Show listing
		MailList($Database, $cgi, $param, $mail);

		# Display success
		Webchat::Table::MakeTop($cgi);
		Webchat::Table::MakeSingle($cgi, $mail->{'TagMailSent'});
		Webchat::Table::MakeBottom($cgi);

	}

	# Imporper puppet name
	else
	{
		# Recall composer
		MailComposer($Database, $cgi, $param, $mail);
	}
}


#####################
# MailViewer
#
# Allows for the viewing of mail to be sent out to the designated person. This
# will allow the puppeteer to reply if necessary as to make life simpler.

sub MailViewer							# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $Database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $user;						# Intended target
	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $body;						# Body of message

	my $id;							# Identification number;


	#####################
	# Program area

	# Identification number
	$id = $param->{'MID'};
	delete $param->{'MID'};

	# Assign OID
	$param->{'OID'} = $id;


	# Retreive information
	($user,
	 $from, 
	 $rcpt, 
	 $subj, 
	 $time, 
	 $body) = Webchat::Database::DataGetPuppeteerMail($Database, $id);

	# Message message row
	$body =~ s/\n/<BR>\n/gm;


	# Only complete if intended target
	if ($user eq $param->{'USER'})
	{
		# Display title
		print "$mail->{'TagTitleViewer'}\n";

		# Display area
		# Start of form, table
		print $cgi->start_form();
		Webchat::Table::MakeTop($cgi);

		# Header
		Webchat::Table::MakeSingle($cgi, $mail->{'TagMailHeader'});
		Webchat::Table::MakeValid($cgi, $mail->{'TagMailFrom'}, $from);
		Webchat::Table::MakeValid($cgi, $mail->{'TagMailTo'}, $rcpt);
		Webchat::Table::MakeValid($cgi, $mail->{'TagMailSubject'}, $subj);
		Webchat::Table::MakeValid($cgi, $mail->{'TagMailDate'}, Webchat::Date::GetDate($time));
			
		# Body
		Webchat::Table::MakeBlank($cgi);
		Webchat::Table::MakeSingle($cgi, $mail->{'TagMailMessage'});
		Webchat::Table::MakeSingle($cgi, "$body");

		# Submit and remove
		Webchat::Table::MakeBlank($cgi);
		Webchat::Table::MakeValid($cgi, $cgi->submit($mail->{'TxtMailReply'}), $cgi->submit($mail->{'TxtMailDelete'}));

		Webchat::Table::MakeBottom($cgi);
		Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'REPLY', $id);
		print $cgi->end_form(), "\n";
	}

	# Invalid component
	else
	{
		# Call listing
		MailList($Database, $cgi, $param, $mail);
	}
}

